### Persefone.jl - a model of agricultural landscapes and ecosystems in Europe.
###
### This file contains code for the fields that farmers manage.
###

mutable struct FarmPlot{T} <: ModelAgent
    const id::Int64
    pixels::Vector{Tuple{Int64, Int64}}
    crop_state :: T
end

croptype(f::FarmPlot{T}) where {T} = croptype(f.crop_state)
cropname(f::FarmPlot{T}) where {T} = cropname(croptype(f))
cropheight(f::FarmPlot{T}) where {T} = cropheight(f.crop_state)
cropcover(f::FarmPlot{T}) where {T} = cropcover(f.crop_state)
cropyield(f::FarmPlot{T}) where {T} = cropyield(f.crop_state)

"""
    initfields_fill_with!(make_farmplot_fn, model)

Initialise the model with its farm plots, using the
`make_farmplot_fn(model, x, y)` function to create new farmplots.
"""
function initfields_fill_with!(make_farmplot_fn::Function, model::SimulationModel)
    n = 0
    convertid = Dict{Int64,Int64}()
    width, height = size(model.landscape)
    for x in 1:width
        for y in 1:height
            # for each pixel, we need to extract the field ID given by the map input
            # file, and convert it into the internal object ID used by Agents.jl,
            # creating a new agent object if necessary
            rawid = model.landscape[x,y].fieldid
            (ismissing(rawid)) && continue
            if rawid in keys(convertid)
                objectid = convertid[rawid]
                model.landscape[x,y].fieldid = objectid
                push!(model.farmplots[objectid].pixels, (x,y))
            else
                fp = make_farmplot_fn(model, x, y)
                push!(model.farmplots, fp)
                model.landscape[x,y].fieldid = fp.id
                convertid[rawid] = fp.id
                n += 1
            end
        end
    end
    @info "Initialised $n farm plots."
end


## UTILITY FUNCTIONS

"""
    averagefieldsize(model)

Calculate the average field size in hectares for the model landscape.
"""
function averagefieldsize(model::SimulationModel)
    conversionfactor = 100 #our pixels are currently 10x10m, so 100 pixels per hectare
    sizes::Vector{Float64} = []
    for fp in model.farmplots
        push!(sizes, size(fp.pixels)[1]/conversionfactor)
    end
    round(sum(sizes)/size(sizes)[1], digits=2)
end

"""
    croptype(model, position)

Return the crop at this position, or nothing if there is no crop here (utility wrapper).
"""
function croptype(pos::Tuple{Int64,Int64}, model::SimulationModel)
    ismissing(model.landscape[pos...].fieldid) ? nothing :
              croptype(model.farmplots[model.landscape[pos...].fieldid])
end

"""
    cropname(model, position)

Return the name of the crop at this position, or an empty string if there is no crop here
(utility wrapper).
"""
function cropname(pos::Tuple{Int64,Int64}, model::SimulationModel)
    field = model.landscape[pos...].fieldid
    ismissing(field) ? "" : cropname(model.farmplots[field])
end

"""
    cropheight(model, position)

Return the height of the crop at this position, or nothing if there is no crop here
(utility wrapper).
"""
function cropheight(pos::Tuple{Int64,Int64}, model::SimulationModel)
    ismissing(model.landscape[pos...].fieldid) ? nothing :
              cropheight(model.farmplots[model.landscape[pos...].fieldid])
end
